SystemOrganization addCategory: #'OB-Web-Core'!
SystemOrganization addCategory: #'OB-Web-Widgets'!

HttpService subclass: #OBHttpService
	instanceVariableNames: 'windows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Core'!

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/18/2006 18:06'!
createLauncher
	| launcher |
	launcher _ OBWebLauncher new.
	windows add: launcher.
	^ launcher initialResponse! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:42'!
createWindowFor: aBrowser
	| window |
	window _ OBWebBuilder build: aBrowser.
	windows add: window.
	^ window! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 20:24'!
handleRequest: aRequest
	| response |
	response _ [(aRequest method = 'GET')
		ifTrue: [self processGet: aRequest]
		ifFalse: [self processPost: aRequest]]
			on: OBBrowseRequest
			do: [:req | req resume: (self createWindowFor: req browser)].
	^ response! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/18/2006 18:03'!
initialize
	super initialize.
	windows _ Set new! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/18/2006 18:02'!
notFound: aRequest
	^ HttpResponse 
		status: #notFound
		contents: aRequest url, ' was not found on this server'! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 20:48'!
processError: anError
	self halt.
	^ HttpResponse basicNew
		status: #serverError;
		contentType: MIMEDocument contentTypePlainText;
		contents: (anError class name, ': ', anError messageText) readStream;
		yourself! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 00:15'!
processGet: aRequest
	
	aRequest url = '/' ifTrue: [^ self createLauncher].
	^ self 
		targetForRequest: aRequest
		ifPresent: [:target | target html]! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 22:19'!
processHttpRequest: aRequest
	^ [self handleRequest: aRequest]
		on: Error
		do: [:error | self processError: error]! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 21:01'!
processPost: aRequest
	| message |
	message _ OBAjaxMessage fromString: aRequest rawPostFields.
	^ self 
		targetForRequest: aRequest
		ifPresent: [:target | Json render: (target processMessage: message)]! !

!OBHttpService methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 23:37'!
targetForRequest: aRequest ifPresent: aBlock
	| id target val |
	id _ aRequest url allButFirst.
	target _ windows
				detect: [:ea | id = ea identifier]
				ifNone: [^ self notFound: aRequest].
	val _ aBlock value: target.
	^ HttpResponse fromString: val! !

Object subclass: #OBAjaxMessage
	instanceVariableNames: 'receiver selector arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Core'!

!OBAjaxMessage class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 20:25'!
fromJson: aJsonObject
	^ self
		receiver: aJsonObject receiver
		selector: aJsonObject selector asSymbol
		arguments: aJsonObject arguments! !

!OBAjaxMessage class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 21:01'!
fromString: aString
	^ self fromJson: (Json readFrom: aString readStream)! !

!OBAjaxMessage class methodsFor: 'as yet unclassified' stamp: 'cwp 10/27/2006 00:28'!
receiver: aString
	^ self basicNew receiver: aString! !

!OBAjaxMessage class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 20:29'!
receiver: receiver selector: selector arguments: arguments
	^ self new 
		setReceiver: receiver 
		selector: selector 
		arguments: arguments! !

!OBAjaxMessage methodsFor: 'accessing' stamp: 'cwp 10/25/2006 21:50'!
arguments
	^ arguments! !

!OBAjaxMessage methodsFor: 'accessing' stamp: 'cwp 10/27/2006 00:29'!
arguments: anArray
	arguments _ anArray! !

!OBAjaxMessage methodsFor: 'receiving' stamp: 'cwp 10/25/2006 20:37'!
isAddressedTo: aWidget
	^ aWidget identifier = receiver! !

!OBAjaxMessage methodsFor: 'sending' stamp: 'cwp 10/26/2006 22:08'!
jsonWriteOn: aStream
	aStream 
		nextPut: ${;
		jsonPrint: 'receiver';
		nextPut: $:;
		jsonPrint: receiver;
		nextPutAll: ', ';
		jsonPrint: 'selector';
		nextPut: $:;
		jsonPrint: selector;
		nextPutAll: ', ';
		jsonPrint: 'arguments';
		nextPut: $:;
		jsonPrint: arguments;
		nextPut: $}! !

!OBAjaxMessage methodsFor: 'sending' stamp: 'cwp 10/26/2006 19:25'!
queue
	(OBAjaxResponse message: self) signal! !

!OBAjaxMessage methodsFor: 'accessing' stamp: 'cwp 10/27/2006 00:29'!
receiver: anObject
	receiver _ anObject! !

!OBAjaxMessage methodsFor: 'accessing' stamp: 'cwp 10/25/2006 21:34'!
selector
	^ selector! !

!OBAjaxMessage methodsFor: 'accessing' stamp: 'cwp 10/27/2006 00:29'!
selector: aSymbol
	selector _ aSymbol! !

!OBAjaxMessage methodsFor: 'receiving' stamp: 'cwp 10/25/2006 21:11'!
sendTo: aWidget
	^ aWidget perform: selector withArguments: arguments! !

!OBAjaxMessage methodsFor: 'initialize-release' stamp: 'cwp 10/25/2006 21:34'!
setReceiver: aString selector: aSymbol arguments: anArray
	receiver _ aString.
	selector _ aSymbol.
	arguments _ anArray! !

Object subclass: #OBWebBuilder
	instanceVariableNames: 'current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Core'!

!OBWebBuilder class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:42'!
build: aBrowser
	| builder |
	builder _ self new.
	aBrowser buildOn: builder.
	^ builder root! !

!OBWebBuilder class methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:55'!
new
	^ self basicNew initialize! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:56'!
addChild: anObject
	current _ anObject! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:51'!
browser: aBrowser with: aBlock
	self 
		current: (OBWebWindow model: aBrowser)
		do: aBlock.
! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 10:13'!
column: aColumn with: aBlock
	self
		current: (OBWebPane model: aColumn)
		do: [self
				current: (OBWebList model: aColumn)
				do: aBlock]! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:51'!
columnPanel: aColumnPanel with: aBlock 
	self
		current: (OBWebScroller model: aColumnPanel)
		do: aBlock! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:57'!
current: aWidget do: aBlock
	| saved |
	current addChild: aWidget.
	saved _ current.
	current _ aWidget.
	aBlock ensure: [current _ saved]! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:52'!
definitionPanel: aPanel with: aBlock
	self 
		current: (OBWebText model: aPanel)
		do: aBlock! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:56'!
initialize
	current _ self
		! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 10:32'!
modalFilter: aFilter with: aBlock
	self
		current: (OBWebButtonBar model: aFilter)
		do: aBlock! !

!OBWebBuilder methodsFor: 'as yet unclassified' stamp: 'cwp 10/23/2006 14:56'!
root
	^ current! !

Object subclass: #OBWebLauncher
	instanceVariableNames: 'id queue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Core'!

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/19/2006 23:53'!
allowedSelectors
	^ #(openBrowser)! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 23:27'!
flushQueue
	| tmp |
	tmp _ queue asArray.
	queue _ OrderedCollection new.
	^ tmp! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 22:40'!
html
	^ '<html>
<head>
	<title>launcher</title>
	<script src="http://localhost/ob/link.js"></script>
	<script src="http://localhost/ob/launcher.js"></script>
</head>
<body onload="initialize()">
	<div id="id1">
		<a href="javascript:launcher.openBrowser()">open browser</a>
	</div>
	<script>
		function initialize()
		{
			link = new Link(window.location);
			launcher = new Launcher("id1");
		}
	</script>
</body>
</html>
'! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/18/2006 18:19'!
id
	^ id! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/18/2006 18:19'!
identifier
	^ id asString! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/18/2006 18:16'!
initialResponse
	^ HttpResponse fromUrl: self path! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 22:16'!
initialize
	id _ UUID new.
	queue _ OrderedCollection new.! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/21/2006 01:12'!
openBrowser
	| window |
	window _ OBSystemBrowser open.
	self queueMessage: (JsonObject new
							receiver: 'id1';
							selector: 'openWindow';
							arguments: {window path};
							yourself)! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/18/2006 18:16'!
path
	^ id asString copyWithFirst: $/! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 23:29'!
processMessage: aMessage
	(self allowedSelectors includes: aMessage selector)
		ifTrue: [aMessage sendTo: self]
		ifFalse: [self webDoesNotUnderstand: aMessage].
	^ self flushQueue! !

!OBWebLauncher methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 22:17'!
queueMessage: aMessageSend
	queue add: aMessageSend! !

Object subclass: #OBWebWidget
	instanceVariableNames: 'id model children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

OBWebWidget subclass: #OBWebButtonBar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

!OBWebButtonBar methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:45'!
htmlClass
	^ 'ButtonBar vertical'! !

OBWebWidget subclass: #OBWebList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 21:36'!
allowedSelectors
	^ #(select:)! !

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:45'!
htmlClass
	^ 'List vertical'! !

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 12:34'!
jsInitializeOn: aStream
	self jsWidgetAccessorOn: aStream.
	aStream 
		nextPutAll: '.update(';
		jsonPrint: self state;
		nextPutAll: ');'! !

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 12:04'!
list
	^ model list! !

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 22:04'!
select: aString
	| index |
	index _ self list indexOf: aString.
	model selection: index.
	! !

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 12:06'!
selection
	| selection |
	selection _ model selection.
	^ selection = 0
		ifTrue: [nil]
		ifFalse: [selection - 1]! !

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 12:36'!
state
	^ JsonObject new
		items: self list;
		selection: self selection;
		yourself! !

!OBWebList methodsFor: 'as yet unclassified' stamp: 'cwp 10/27/2006 00:35'!
update: aSelector
	aSelector = #list ifTrue: 
		[self createMessage
			selector: #update;
			arguments: {self state};
			queue]! !

OBWebWidget subclass: #OBWebPane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

!OBWebPane methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:45'!
htmlClass
	^ 'Pane horizontal'! !

OBWebWidget subclass: #OBWebScroller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

!OBWebScroller methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:44'!
htmlClass
	^ 'Scroller vertical'! !

!OBWebScroller methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 13:28'!
htmlStyle
	^ 'top: 0%; height: 40%'! !

OBWebWidget subclass: #OBWebText
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

!OBWebText methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:45'!
htmlClass
	^ 'Text vertical'! !

!OBWebText methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 15:31'!
htmlStyle
	^ 'top: 40%; height: 60%'! !

!OBWebWidget class methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 14:44'!
id: aWebIdentifier model: anObject
	^ (self basicNew initializeWithId: aWebIdentifier)
		model: anObject;
		yourself! !

!OBWebWidget class methodsFor: 'as yet unclassified' stamp: 'cwp 10/22/2006 02:10'!
model: aBrowser
	^ self new model: aBrowser! !

!OBWebWidget class methodsFor: 'as yet unclassified' stamp: 'cwp 10/22/2006 02:10'!
new
	^ self basicNew initialize! !

!OBWebWidget methodsFor: 'children' stamp: 'cwp 10/25/2006 19:08'!
addAllChildrenTo: aDictionary
	children do: [:ea | ea addWithAllChildrenTo: aDictionary].
	^ aDictionary! !

!OBWebWidget methodsFor: 'children' stamp: 'cwp 10/22/2006 02:09'!
addChild: aWidget
	children add: aWidget! !

!OBWebWidget methodsFor: 'children' stamp: 'cwp 10/25/2006 19:08'!
addWithAllChildrenTo: aDictionary
	aDictionary at: self identifier put: self.
	^ self addAllChildrenTo: aDictionary.
	! !

!OBWebWidget methodsFor: 'children' stamp: 'cwp 10/25/2006 19:20'!
allChildren
	^ self addAllChildrenTo: (Dictionary new)! !

!OBWebWidget methodsFor: 'accessing' stamp: 'cwp 10/25/2006 21:38'!
allowedSelectors
	^ #()! !

!OBWebWidget methodsFor: 'children' stamp: 'cwp 10/22/2006 01:50'!
children
	^ children! !

!OBWebWidget methodsFor: 'accessing' stamp: 'cwp 10/27/2006 00:26'!
createMessage
	^ OBAjaxMessage receiver: self identifier! !

!OBWebWidget methodsFor: 'ajax' stamp: 'cwp 10/25/2006 18:57'!
handleMessage: message
	| selector |
	selector _ message selector asSymbol.
	^ (self allowedSelectors includes: selector)
		ifTrue: [self perform: selector withArguments: message arguments. true]
		ifFalse: [false]! !

!OBWebWidget methodsFor: 'html' stamp: 'cwp 10/24/2006 14:52'!
html
	^ String streamContents: [:s | self htmlOn: s]! !

!OBWebWidget methodsFor: 'html' stamp: 'cwp 10/24/2006 15:31'!
htmlStyle
	^ nil! !

!OBWebWidget methodsFor: 'html' stamp: 'cwp 10/24/2006 15:31'!
htmlTagOn: aStream
	aStream 
		nextPutAll: '<div id="';
		print: id;
		nextPutAll: '" class="';
		nextPutAll: self htmlClass;
		nextPut: $".
	self htmlStyle ifNotNilDo: 
		[:style | aStream
					nextPutAll: ' style="';
					nextPutAll: style;
					nextPut: $"].
	aStream nextPut: $>.
	children do: [:ea | ea htmlTagOn: aStream].
	aStream nextPutAll: '</div>'! !

!OBWebWidget methodsFor: 'accessing' stamp: 'cwp 10/22/2006 01:49'!
identifier
	^ id asString! !

!OBWebWidget methodsFor: 'initialize-release' stamp: 'cwp 10/24/2006 14:31'!
initialize
	self initializeWithId: (OBWebIdentifier new)! !

!OBWebWidget methodsFor: 'initialize-release' stamp: 'cwp 10/24/2006 14:30'!
initializeWithId: aWebIdentifier
	id _ aWebIdentifier.
	children _ OrderedCollection new.! !

!OBWebWidget methodsFor: 'html' stamp: 'cwp 10/25/2006 12:08'!
jsInitializeOn: aStream
	children 
		do: [:ea | ea jsInitializeOn: aStream]
		separatedBy: [aStream space]! !

!OBWebWidget methodsFor: 'html' stamp: 'cwp 10/25/2006 12:03'!
jsWidgetAccessorOn: aStream
	aStream
		nextPutAll: 'Widget.get("';
		print: id;
		nextPutAll: '")'! !

!OBWebWidget methodsFor: 'accessing' stamp: 'cwp 10/22/2006 01:49'!
model
	^ model! !

!OBWebWidget methodsFor: 'accessing' stamp: 'cwp 10/25/2006 22:10'!
model: anObject
	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].
	model _ anObject! !

!OBWebWidget methodsFor: 'initialize-release' stamp: 'cwp 10/25/2006 13:42'!
registerWith: aHttpService
	aHttpService addResponder: self.
	children do: [:ea | ea registerWith: aHttpService]! !

!OBWebWidget methodsFor: 'accessing' stamp: 'cwp 10/27/2006 00:11'!
update: aSelector
	Transcript cr; show: self class name, ': ', aSelector! !

!OBWebWidget methodsFor: 'children' stamp: 'cwp 10/25/2006 19:20'!
withAllChildren
	^ self addWithAllChildrenTo: Dictionary new! !

OBWebWidget subclass: #OBWebWindow
	instanceVariableNames: 'queue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 20:03'!
flushQueue
	| tmp |
	tmp _ queue.
	queue _ OrderedCollection new.
	^ tmp! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/27/2006 01:00'!
htmlClass
	^ 'Root root'! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 11:48'!
htmlOn: aStream
	aStream nextPutAll: '<html><head><title>browser</title><script src="http://localhost/ob/link.js"></script><script src="http://localhost/ob/widgets.js"></script><link rel="stylesheet" href="http://localhost/ob/browser.css" type="text/css" /></head><body>'.
		self htmlTagOn: aStream.
	aStream nextPutAll: '<script>'.
	self jsInitializeOn: aStream.
	aStream
		nextPutAll: '</script>';
		nextPutAll: '</body></html>'
! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 20:00'!
initializeWithId: aWebIdentifier
	super initializeWithId: aWebIdentifier.
	queue _ OrderedCollection new! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 13:13'!
jsInitializeOn: aStream
	aStream 
		nextPutAll: 'Widget.init("';
		print: id;
		nextPutAll: '"); '.
	super jsInitializeOn: aStream.! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/20/2006 00:32'!
path
	^ self identifier copyWithFirst: $/! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 19:56'!
processMessage: message
	| table receiver |
	table _ self withAllChildren.
	receiver _ table 
				detect: [:ea | message isAddressedTo: ea] 
				ifNone: [^ 'Unknown receiver'].
	[receiver handleMessage: message]
		on: OBAjaxResponse
		do: [:response | self queueMessage: response message].
	^ self flushQueue! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 19:59'!
queueMessage: anAjaxMessage
	queue add: anAjaxMessage! !

!OBWebWindow methodsFor: 'as yet unclassified' stamp: 'cwp 10/27/2006 01:01'!
update: aSelector
	aSelector = #relabel
		ifTrue: [self createMessage
					selector: 'title';
					arguments: {model labelString};
					queue]! !

OBInteractionRequest subclass: #OBAjaxResponse
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Core'!

!OBAjaxResponse class methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 19:23'!
message: anAjaxMessage
	^ self new setMessage: anAjaxMessage! !

!OBAjaxResponse methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 20:56'!
message
	^ message! !

!OBAjaxResponse methodsFor: 'as yet unclassified' stamp: 'cwp 10/26/2006 19:20'!
setMessage: anAjaxMessage
	message _ anAjaxMessage! !

!HttpRequest methodsFor: '*ob-web-accessing' stamp: 'cwp 10/19/2006 23:38'!
postData
	| rawPostFields |
	rawPostFields _ self rawPostFields.
	^ rawPostFields isEmpty
		ifTrue: [stream next: self contentLength]
		ifFalse: [rawPostFields]! !

LargePositiveInteger variableByteSubclass: #OBWebIdentifier
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OB-Web-Widgets'!

!OBWebIdentifier class methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 11:23'!
defaultSize
	^ 16! !

!OBWebIdentifier class methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 11:22'!
new
	^ self new: self defaultSize! !

!OBWebIdentifier class methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 11:22'!
random: aRandom
	^ (self basicNew: self defaultSize) initializeWithRandom: aRandom! !

!OBWebIdentifier class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2006 13:51'!
seed: anInteger
	^ self random: (Random seed: anInteger)! !

!OBWebIdentifier methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 11:11'!
initialize
	self initializeWithRandom: Random new! !

!OBWebIdentifier methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 11:15'!
initializeWithRandom: aRandom
	1 to: self size do: [:i | self at: i put: (aRandom nextInt: 255)]! !

!OBWebIdentifier methodsFor: 'as yet unclassified' stamp: 'cwp 10/24/2006 11:42'!
printOn: aStream
	
	| base digits current i letters quo |
	base _ 36.
	digits _ '0123456789abcdefghijklmnopqrstuvwxyz'.
	current _ self.
	i _ self digitLength * 8.
	letters _ String new: i.
	[current < base] whileFalse: 
		[quo _ current quo: base.
		letters at: i put: (digits at: (current - (quo * base)) + 1).
		i _ i - 1.
		current _ quo].
	letters at: i put: (digits at: current + 1).
	aStream nextPutAll: (letters copyFrom: i to: letters size)
	
! !
